import pytest

from nypl_py_utils import PostgreSQLClient, PostgreSQLClientError
from psycopg import Error


class TestPostgreSQLClient:

    @pytest.fixture
    def test_instance(self, mocker):
        mocker.patch('psycopg_pool.ConnectionPool.open')
        mocker.patch('psycopg_pool.ConnectionPool.close')
        return PostgreSQLClient('test_host', 'test_port', 'test_db_name',
                                'test_user', 'test_password')

    def test_init(self, test_instance):
        assert test_instance.pool.conninfo == (
            'postgresql://test_user:test_password@test_host:test_port/' +
            'test_db_name')
        assert test_instance.pool._opened is False
        assert test_instance.pool.min_size == 0
        assert test_instance.pool.max_size == 1

    def test_init_with_kwargs(self):
        test_instance = PostgreSQLClient(
            'test_host', 'test_port', 'test_db_name', 'test_user',
            'test_password', min_size=5, max_size=10)
        assert test_instance.pool.conninfo == (
            'postgresql://test_user:test_password@test_host:test_port/' +
            'test_db_name')
        assert test_instance.pool._opened is False
        assert test_instance.pool.min_size == 5
        assert test_instance.pool.max_size == 10

    def test_connect(self, test_instance):
        test_instance.connect()
        test_instance.pool.open.assert_called_once_with(wait=True, timeout=300)

    def test_connect_with_exception(self, mocker):
        mocker.patch('psycopg_pool.ConnectionPool.open',
                     side_effect=Error())

        test_instance = PostgreSQLClient(
            'test_host', 'test_port', 'test_db_name', 'test_user',
            'test_password', timeout=1.0)

        with pytest.raises(PostgreSQLClientError):
            test_instance.connect()

    def test_execute_read_query(self, test_instance, mocker):
        test_instance.connect()

        mock_cursor = mocker.MagicMock()
        mock_cursor.fetchall.return_value = [(1, 2, 3), ('a', 'b', 'c')]
        mock_conn = mocker.MagicMock()
        mock_conn.execute.return_value = mock_cursor
        mock_conn_context = mocker.MagicMock()
        mock_conn_context.__enter__.return_value = mock_conn
        mocker.patch('psycopg_pool.ConnectionPool.connection',
                     return_value=mock_conn_context)

        assert test_instance.execute_query(
            'test query') == [(1, 2, 3), ('a', 'b', 'c')]
        mock_conn.execute.assert_called_once_with('test query', None)
        mock_cursor.fetchall.assert_called_once()

    def test_execute_write_query(self, test_instance, mocker):
        test_instance.connect()

        mock_conn = mocker.MagicMock()
        mock_conn_context = mocker.MagicMock()
        mock_conn_context.__enter__.return_value = mock_conn
        mocker.patch('psycopg_pool.ConnectionPool.connection',
                     return_value=mock_conn_context)

        assert test_instance.execute_query(
            'test query', is_write_query=True) is None
        mock_conn.execute.assert_called_once_with('test query', None)
        mock_conn.commit.assert_called_once()

    def test_execute_write_query_with_params(self, test_instance, mocker):
        test_instance.connect()

        mock_conn = mocker.MagicMock()
        mock_conn_context = mocker.MagicMock()
        mock_conn_context.__enter__.return_value = mock_conn
        mocker.patch('psycopg_pool.ConnectionPool.connection',
                     return_value=mock_conn_context)

        assert test_instance.execute_query(
            'test query %s %s', is_write_query=True,
            query_params=('a', 1)) is None
        mock_conn.execute.assert_called_once_with('test query %s %s',
                                                  ('a', 1))
        mock_conn.commit.assert_called_once()

    def test_execute_query_with_exception(self, test_instance, mocker):
        test_instance.connect()

        mock_conn = mocker.MagicMock()
        mock_conn.execute.side_effect = Exception()
        mock_conn_context = mocker.MagicMock()
        mock_conn_context.__enter__.return_value = mock_conn
        mocker.patch('psycopg_pool.ConnectionPool.connection',
                     return_value=mock_conn_context)

        with pytest.raises(PostgreSQLClientError):
            test_instance.execute_query('test query')

        mock_conn.rollback.assert_called_once()

    def test_close_connection(self, test_instance):
        test_instance.connect()
        test_instance.close_connection()
        assert test_instance.pool is None

    def test_reopen_connection(self, test_instance, mocker):
        test_instance.connect()
        test_instance.close_connection()
        test_instance.connect()
        test_instance.pool.open.assert_has_calls([
            mocker.call(wait=True, timeout=300),
            mocker.call(wait=True, timeout=300)])



import json
import pytest

from nypl_py_utils import S3Client

_TEST_STATE = {'key1': 'val1', 'key2': 'val2', 'key3': 'val3'}


class TestS3Client:

    @pytest.fixture
    def test_instance(self, mocker):
        mocker.patch('boto3.client')
        return S3Client('test_s3_bucket', 'test_s3_resource')

    def test_fetch_cache(self, test_instance):
        def mock_download(bucket, resource, stream):
            assert bucket == 'test_s3_bucket'
            assert resource == 'test_s3_resource'
            stream.write(json.dumps(_TEST_STATE).encode())

        test_instance.s3_client.download_fileobj.side_effect = mock_download
        assert test_instance.fetch_cache() == _TEST_STATE

    def test_set_cache(self, test_instance):
        test_instance.set_cache(_TEST_STATE)
        arguments = test_instance.s3_client.upload_fileobj.call_args.args
        assert arguments[0].getvalue() == json.dumps(_TEST_STATE).encode()
        assert arguments[1] == 'test_s3_bucket'
        assert arguments[2] == 'test_s3_resource'



import avro.schema
import requests

from avro.errors import AvroException
from avro.io import BinaryDecoder, BinaryEncoder, DatumReader, DatumWriter
from io import BytesIO
from nypl_py_utils.functions.log_helper import create_log
from requests.exceptions import JSONDecodeError, RequestException


class AvroEncoder:
    """
    Class for encoding records using an Avro schema. Takes as input the
    Platform API endpoint from which to fetch the schema in JSON format.
    """

    def __init__(self, platform_schema_url):
        self.logger = create_log('avro_encoder')
        self.schema = avro.schema.parse(
            self._get_json_schema(platform_schema_url))

    def encode_record(self, record):
        """
        Encodes a single JSON record using the given Avro schema.

        Returns the encoded record as a byte string.
        """
        self.logger.debug(
            'Encoding record using {schema} schema'.format(
               schema=self.schema.name))
        datum_writer = DatumWriter(self.schema)
        with BytesIO() as output_stream:
            encoder = BinaryEncoder(output_stream)
            try:
                datum_writer.write(record, encoder)
                return output_stream.getvalue()
            except AvroException as e:
                self.logger.error('Failed to encode record: {}'.format(e))
                raise AvroEncoderError(
                    'Failed to encode record: {}'.format(e)) from None

    def encode_batch(self, record_list):
        """
        Encodes a list of JSON records using the given Avro schema.

        Returns a list of byte strings where each string is an encoded record.
        """
        self.logger.info(
            'Encoding ({num_rec}) records using {schema} schema'.format(
                num_rec=len(record_list), schema=self.schema.name))
        encoded_records = []
        datum_writer = DatumWriter(self.schema)
        with BytesIO() as output_stream:
            encoder = BinaryEncoder(output_stream)
            for record in record_list:
                try:
                    datum_writer.write(record, encoder)
                    encoded_records.append(output_stream.getvalue())
                    output_stream.seek(0)
                    output_stream.truncate(0)
                except AvroException as e:
                    self.logger.error('Failed to encode record: {}'.format(e))
                    raise AvroEncoderError(
                        'Failed to encode record: {}'.format(e)) from None
        return encoded_records

    def decode_record(self, record):
        """
        Decodes a single record represented as a byte string using the given
        Avro schema.

        Returns a dictionary where each key is a field in the schema.
        """
        self.logger.debug('Decoding {rec} using {schema} schema'.format(
            rec=record, schema=self.schema.name))
        datum_reader = DatumReader(self.schema)
        with BytesIO(record) as input_stream:
            decoder = BinaryDecoder(input_stream)
            try:
                return datum_reader.read(decoder)
            except Exception as e:
                self.logger.error('Failed to decode record: {}'.format(e))
                raise AvroEncoderError(
                    'Failed to decode record: {}'.format(e)) from None

    def _get_json_schema(self, platform_schema_url):
        """
        Fetches a JSON response from the input Platform API endpoint and
        interprets it as an Avro schema.
        """
        self.logger.info('Fetching Avro schema from {}'.format(
            platform_schema_url))
        try:
            response = requests.get(platform_schema_url)
            response.raise_for_status()
        except RequestException as e:
            self.logger.error(
                'Failed to retrieve schema from {url}: {error}'.format(
                    url=platform_schema_url, error=e))
            raise AvroEncoderError(
                'Failed to retrieve schema from {url}: {error}'.format(
                    url=platform_schema_url, error=e)) from None

        try:
            json_response = response.json()
            return json_response['data']['schema']
        except (JSONDecodeError, KeyError) as e:
            self.logger.error(
                'Retrieved schema is malformed: {errorType} {errorMessage}'
                .format(errorType=type(e), errorMessage=e))
            raise AvroEncoderError(
                'Retrieved schema is malformed: {errorType} {errorMessage}'
                .format(errorType=type(e), errorMessage=e)) from None


class AvroEncoderError(Exception):
    def __init__(self, message=None):
        self.message = message



import psycopg

from nypl_py_utils.functions.log_helper import create_log
from psycopg.rows import tuple_row
from psycopg_pool import ConnectionPool


class PostgreSQLClient:
    """
    Client for managing connections to a PostgreSQL database (such as Sierra)
    """

    def __init__(self, host, port, db_name, user, password, **kwargs):
        self.logger = create_log('postgresql_client')
        self.db_name = db_name
        self.timeout = kwargs.get('timeout', 300)

        self.conn_info = ('postgresql://{user}:{password}@{host}:{port}/'
                          '{db_name}').format(user=user, password=password,
                                              host=host, port=port,
                                              db_name=db_name)
        self.min_size = kwargs.get('min_size', 0)
        self.max_size = kwargs.get('max_size', 1)
        self.pool = ConnectionPool(
            self.conn_info, open=False,
            min_size=self.min_size, max_size=self.max_size)

    def connect(self):
        """
        Opens the connection pool and connects to the given PostgreSQL database
        min_size number of times.
        """
        self.logger.info('Connecting to {} database'.format(self.db_name))
        try:
            if self.pool is None:
                self.pool = ConnectionPool(
                    self.conn_info, open=False, min_size=self.min_size,
                    max_size=self.max_size)
            self.pool.open(wait=True, timeout=self.timeout)
        except psycopg.Error as e:
            self.logger.error(
                'Error connecting to {name} database: {error}'.format(
                    name=self.db_name, error=e))
            raise PostgreSQLClientError(
                'Error connecting to {name} database: {error}'.format(
                    name=self.db_name, error=e)) from None

    def execute_query(self, query, is_write_query=False, query_params=None,
                      row_factory=tuple_row):
        """
        Requests a connection from the pool and uses it to execute an arbitrary
        query. After the query is complete, returns the connection to the pool.

        Parameters
        ----------
        query: str
            The query to execute
        is_write_query: bool, optional
            Whether or not the query is writing to the database, in which case
            the transaction needs to be committed and None should be returned
        query_params: sequence, optional
            The values to be used in a parameterized query
        row_factory: RowFactory, optional
            A psycopg RowFactory that determines how the data will be returned.
            Defaults to tuple_row, which returns the rows as a list of tuples.

        Returns
        -------
        None or sequence
            None if is_write_query is True. Some type of sequence based on
            the row_factory input if is_write_query is False.
        """
        self.logger.info('Querying {} database'.format(self.db_name))
        self.logger.debug('Executing query {}'.format(query))
        with self.pool.connection() as conn:
            try:
                conn.row_factory = row_factory
                cursor = conn.execute(query, query_params)
                if is_write_query:
                    conn.commit()
                    return None
                else:
                    return cursor.fetchall()
            except Exception as e:
                conn.rollback()
                self.logger.error(
                    ('Error executing {name} database query \'{query}\': '
                     '{error}').format(
                        name=self.db_name, query=query, error=e))
                raise PostgreSQLClientError(
                    ('Error executing {name} database query \'{query}\': '
                     '{error}').format(
                        name=self.db_name, query=query, error=e)) from None

    def close_connection(self):
        """Closes the connection pool"""
        self.logger.debug('Closing {} database connection'.format(
            self.db_name))
        self.pool.close()
        self.pool = None


class PostgreSQLClientError(Exception):
    def __init__(self, message=None):
        self.message = message



from .classes.avro_encoder import AvroEncoder, AvroEncoderError  # noqa
from .classes.kinesis_client import KinesisClient, KinesisClientError  # noqa
from .classes.kms_client import KmsClient, KmsClientError # noqa
from .classes.mysql_client import MySQLClient, MySQLClientError  # noqa
from .classes.oauth2_api_client import Oauth2ApiClient  # noqa
from .classes.postgresql_client import PostgreSQLClient, PostgreSQLClientError  # noqa
from .classes.redshift_client import RedshiftClient, RedshiftClientError  # noqa
from .classes.s3_client import S3Client, S3ClientError  # noqa



import os
import json
import pytest
from requests_oauthlib import OAuth2Session

from nypl_py_utils import Oauth2ApiClient
# from requests.exceptions import ConnectTimeout

_TOKEN_RESPONSE = {
    'access_token': 'super-secret-token',
    'expires_in': 3600,
    'token_type': 'Bearer',
    'scope': ['offline_access', 'openid', 'login:staff', 'admin'],
    'id_token': 'super-secret-token',
    'expires_at': 1677599823.3180869
}

BASE_URL = 'https://example.com/api/v0.1'


class TestOauth2ApiClient:

    @pytest.fixture
    def test_instance(self, requests_mock):
        token_url = 'https://oauth.example.com/oauth/token'
        requests_mock.post(token_url, text=json.dumps(_TOKEN_RESPONSE))

        return Oauth2ApiClient(base_url=BASE_URL,
                               token_url=token_url,
                               client_id='clientid',
                               client_secret='clientsecret'
                               )

    def test_uses_env_vars(self):
        env = {
            'NYPL_API_CLIENT_ID': 'env client id',
            'NYPL_API_CLIENT_SECRET': 'env client secret',
            'NYPL_API_TOKEN_URL': 'env token url',
            'NYPL_API_BASE_URL': 'env base url'
        }
        for key, value in env.items():
            os.environ[key] = value

        client = Oauth2ApiClient()
        assert client.client_id == 'env client id'
        assert client.client_secret == 'env client secret'
        assert client.token_url == 'env token url'
        assert client.base_url == 'env base url'

        for key, value in env.items():
            os.environ[key] = ''

    def test_generate_access_token(self, test_instance):
        test_instance._generate_access_token()
        assert test_instance.token['access_token']\
            == _TOKEN_RESPONSE['access_token']

    def test_create_oauth_client(self, test_instance):
        test_instance._create_oauth_client()
        assert type(test_instance.oauth_client) is OAuth2Session

    def test_do_http_method(self, requests_mock, test_instance):
        requests_mock.get(f'{BASE_URL}/foo', json={'foo': 'bar'})
        resp = test_instance._do_http_method('GET', 'foo')
        assert resp == {'foo': 'bar'}

    def test_token_expiration(self, requests_mock, test_instance):
        api_get_mock = requests_mock.get(f'{BASE_URL}/foo',
                                         json={'foo': 'bar'})

        # Perform first request to auto-authenticate:
        resp = test_instance._do_http_method('GET', 'foo')
        assert api_get_mock.request_history[0]._request\
            .headers['Authorization'] == 'Bearer super-secret-token'

        # Emulate token expiration:
        test_instance.token['expires_at'] = 0

        token_post_mock = requests_mock.post(
            test_instance.token_url,
            text=json.dumps(_TOKEN_RESPONSE)
        )

        # Perform second request, which should detect token expiration and
        # re-authenticate:
        resp = test_instance._do_http_method('GET', 'foo')

        assert token_post_mock.called is True
        assert api_get_mock.request_history[1]._request\
            .headers['Authorization'] == 'Bearer super-secret-token'
        assert resp == {'foo': 'bar'}



import mysql.connector

from nypl_py_utils.functions.log_helper import create_log


class MySQLClient:
    """Client for managing connections to a MySQL database"""

    def __init__(self, host, port, database, user, password):
        self.logger = create_log('mysql_client')
        self.conn = None
        self.host = host
        self.port = port
        self.database = database
        self.user = user
        self.password = password

    def connect(self):
        """Connects to a MySQL database using the given credentials"""
        self.logger.info('Connecting to {} database'.format(self.database))
        try:
            self.conn = mysql.connector.connect(
                host=self.host,
                port=self.port,
                database=self.database,
                user=self.user,
                password=self.password)
        except mysql.connector.Error as e:
            self.logger.error(
                'Error connecting to {name} database: {error}'.format(
                    name=self.database, error=e))
            raise MySQLClientError(
                'Error connecting to {name} database: {error}'.format(
                    name=self.database, error=e)) from None

    def execute_query(self, query, is_write_query=False, query_params=None,
                      dictionary=False):
        """
        Executes an arbitrary query against the given database connection.

        Parameters
        ----------
        query: str
            The query to execute
        is_write_query: bool, optional
            Whether or not the query is writing to the database, in which case
            the transaction needs to be committed and None should be returned
        query_params: sequence, optional
            The values to be used in a parameterized query
        dictionary: bool, optional
            Whether the data will be returned as a dictionary. Defaults to
            False, which means the data is returned as a list of tuples.

        Returns
        -------
        None or sequence
            None if is_write_query is True. A list of either tuples or
            dictionaries (based on the dictionary input) if is_write_query is
            False.
        """
        self.logger.info('Querying {} database'.format(self.database))
        self.logger.debug('Executing query {}'.format(query))
        try:
            cursor = self.conn.cursor(dictionary=dictionary)
            cursor.execute(query, query_params)
            if is_write_query:
                self.conn.commit()
                return None
            else:
                return cursor.fetchall()
        except Exception as e:
            self.conn.rollback()
            self.logger.error(
                ('Error executing {name} database query \'{query}\': {error}')
                .format(name=self.database, query=query, error=e))
            raise MySQLClientError(
                ('Error executing {name} database query \'{query}\': {error}')
                .format(name=self.database, query=query, error=e)) from None
        finally:
            cursor.close()

    def close_connection(self):
        """Closes the database connection"""
        self.logger.debug('Closing {} database connection'.format(
            self.database))
        self.conn.close()


class MySQLClientError(Exception):
    def __init__(self, message=None):
        self.message = message



import pytest

from freezegun import freeze_time
from nypl_py_utils import KinesisClient, KinesisClientError

_TEST_DATETIME_KEY = '1672531200000000000'
_TEST_KINESIS_RECORDS = [
    {'Data': b'a', 'PartitionKey': _TEST_DATETIME_KEY},
    {'Data': b'b', 'PartitionKey': _TEST_DATETIME_KEY},
    {'Data': b'c', 'PartitionKey': _TEST_DATETIME_KEY},
    {'Data': b'd', 'PartitionKey': _TEST_DATETIME_KEY},
    {'Data': b'e', 'PartitionKey': _TEST_DATETIME_KEY}
]


@freeze_time('2023-01-01')
class TestKinesisClient:

    @pytest.fixture
    def test_instance(self, mocker):
        mocker.patch('boto3.client')
        return KinesisClient('test_stream_arn', 2)

    def test_send_records(self, test_instance, mocker):
        MOCK_RECORDS = [b'a', b'b', b'c', b'd', b'e']
        mocked_send_method = mocker.patch(
            'nypl_py_utils.classes.kinesis_client.KinesisClient._send_kinesis_format_records')  # noqa: E501

        test_instance.send_records(MOCK_RECORDS)
        mocked_send_method.assert_has_calls([
            mocker.call([_TEST_KINESIS_RECORDS[0],
                        _TEST_KINESIS_RECORDS[1]], 1),
            mocker.call([_TEST_KINESIS_RECORDS[2],
                        _TEST_KINESIS_RECORDS[3]], 1),
            mocker.call([_TEST_KINESIS_RECORDS[4]], 1)])

    def test_send_kinesis_format_records(self, test_instance):
        test_instance.kinesis_client.put_records.return_value = {
            'FailedRecordCount': 0}

        test_instance._send_kinesis_format_records(_TEST_KINESIS_RECORDS, 1)
        test_instance.kinesis_client.put_records.assert_called_once_with(
            Records=_TEST_KINESIS_RECORDS, StreamARN='test_stream_arn')

    def test_send_kinesis_format_records_with_failures(
            self, test_instance, mocker):
        test_instance.kinesis_client.put_records.side_effect = [
            {'FailedRecordCount': 2, 'Records': [
                'record0', {'ErrorCode': 1},
                'record2', {'ErrorCode': 3},
                'record4']},
            {'FailedRecordCount': 0}]

        test_instance._send_kinesis_format_records(_TEST_KINESIS_RECORDS, 1)
        test_instance.kinesis_client.put_records.assert_has_calls([
            mocker.call(Records=_TEST_KINESIS_RECORDS,
                        StreamARN='test_stream_arn'),
            mocker.call(Records=[_TEST_KINESIS_RECORDS[1],
                                 _TEST_KINESIS_RECORDS[3]],
                        StreamARN='test_stream_arn')])

    def test_send_kinesis_format_records_with_repeating_failures(
            self, test_instance, mocker):
        test_instance.kinesis_client.put_records.side_effect = [
            {'FailedRecordCount': 5, 'Records': [
                {'ErrorCode': 0}, {'ErrorCode': 1}, {'ErrorCode': 2},
                {'ErrorCode': 3}, {'ErrorCode': 4}]}] * 5

        with pytest.raises(KinesisClientError):
            test_instance._send_kinesis_format_records(
                _TEST_KINESIS_RECORDS, 1)

        test_instance.kinesis_client.put_records.assert_has_calls([
            mocker.call(Records=_TEST_KINESIS_RECORDS,
                        StreamARN='test_stream_arn')] * 5)



import logging
import os
import time

from freezegun import freeze_time
from nypl_py_utils.functions.log_helper import create_log


@freeze_time('2023-01-01 19:00:00')
class TestLogHelper:

    def test_default_logging(self, caplog):
        logger = create_log('test_log')
        assert logger.getEffectiveLevel() == logging.INFO
        assert len(logger.handlers) == 1

        logger.info('Test info message')
        # freeze_time changes the utc time, while the logger uses local time by
        # default, so force the logger to use utc time
        logger.handlers[0].formatter.converter = time.gmtime
        assert len(caplog.records) == 1
        assert logger.handlers[0].format(caplog.records[0]) == \
            '2023-01-01 19:00:00,000 | test_log | INFO: Test info message'

    def test_logging_with_custom_log_level(self, caplog):
        os.environ['LOG_LEVEL'] = 'error'
        logger = create_log('test_log')
        assert logger.getEffectiveLevel() == logging.ERROR

        logger.info('Test info message')
        logger.error('Test error message')
        assert len(caplog.records) == 1
        # freeze_time changes the utc time, while the logger uses local time by
        # default, so force the logger to use utc time
        logger.handlers[0].formatter.converter = time.gmtime
        assert logger.handlers[0].format(caplog.records[0]) == \
            '2023-01-01 19:00:00,000 | test_log | ERROR: Test error message'
        del os.environ['LOG_LEVEL']

    def test_logging_no_duplicates(self, caplog):
        logger = create_log('test_log')
        logger.info('Test info message')

        # Test that logger uses the most recently set log level and doesn't
        # duplicate handlers/messages when create_log is called more than once.
        os.environ['LOG_LEVEL'] = 'error'
        logger = create_log('test_log')
        assert logger.getEffectiveLevel() == logging.ERROR
        assert len(logger.handlers) == 1

        logger.info('Test info message 2')
        logger.error('Test error message')
        assert len(caplog.records) == 2
        # freeze_time changes the utc time, while the logger uses local time by
        # default, so force the logger to use utc time
        logger.handlers[0].formatter.converter = time.gmtime
        assert logger.handlers[0].format(caplog.records[0]) == \
            '2023-01-01 19:00:00,000 | test_log | INFO: Test info message'
        assert logger.handlers[0].format(caplog.records[1]) == \
            '2023-01-01 19:00:00,000 | test_log | ERROR: Test error message'
        del os.environ['LOG_LEVEL']



import pytest

from nypl_py_utils import RedshiftClient, RedshiftClientError


class TestRedshiftClient:

    @pytest.fixture
    def mock_redshift_conn(self, mocker):
        return mocker.patch('redshift_connector.connect')

    @pytest.fixture
    def test_instance(self):
        return RedshiftClient('test_host', 'test_database', 'test_user',
                              'test_password')

    def test_connect(self, mock_redshift_conn, test_instance):
        test_instance.connect()
        mock_redshift_conn.assert_called_once_with(host='test_host',
                                                   database='test_database',
                                                   user='test_user',
                                                   password='test_password')

    def test_execute_query(self, mock_redshift_conn, test_instance, mocker):
        test_instance.connect()

        mock_cursor = mocker.MagicMock()
        mock_cursor.fetchall.return_value = [[1, 2, 3], ['a', 'b', 'c']]
        test_instance.conn.cursor.return_value = mock_cursor

        assert test_instance.execute_query(
            'test query') == [[1, 2, 3], ['a', 'b', 'c']]
        mock_cursor.execute.assert_called_once_with('test query')
        mock_cursor.fetchall.assert_called_once()
        mock_cursor.close.assert_called_once()

    def test_execute_dataframe_query(self, mock_redshift_conn, test_instance,
                                     mocker):
        test_instance.connect()

        mock_cursor = mocker.MagicMock()
        test_instance.conn.cursor.return_value = mock_cursor

        test_instance.execute_query('test query', dataframe=True)
        mock_cursor.execute.assert_called_once_with('test query')
        mock_cursor.fetch_dataframe.assert_called_once()
        mock_cursor.close.assert_called_once()

    def test_execute_query_with_exception(
            self, mock_redshift_conn, test_instance, mocker):
        test_instance.connect()

        mock_cursor = mocker.MagicMock()
        mock_cursor.execute.side_effect = Exception()
        test_instance.conn.cursor.return_value = mock_cursor

        with pytest.raises(RedshiftClientError):
            test_instance.execute_query('test query')

        test_instance.conn.rollback.assert_called_once()
        mock_cursor.close.assert_called_once()

    def test_close_connection(self, mock_redshift_conn, test_instance):
        test_instance.connect()
        test_instance.close_connection()
        test_instance.conn.close.assert_called_once()



import logging
import os
import sys

levels = {
    'debug': logging.DEBUG,
    'info': logging.INFO,
    'warning': logging.WARNING,
    'error': logging.ERROR,
    'critical': logging.CRITICAL
}


def create_log(module):
    logger = logging.getLogger(module)
    if logger.hasHandlers():
        logger.handlers = []

    console_log = logging.StreamHandler(stream=sys.stdout)

    log_level = os.environ.get('LOG_LEVEL', 'info').lower()

    logger.setLevel(levels[log_level])
    console_log.setLevel(levels[log_level])

    formatter = logging.Formatter(
        '%(asctime)s | %(name)s | %(levelname)s: %(message)s')
    console_log.setFormatter(formatter)

    logger.addHandler(console_log)

    return logger



import boto3
import os
import time

from botocore.exceptions import ClientError
from nypl_py_utils.functions.log_helper import create_log


class KinesisClient:
    """
    Client for sending records to AWS Kinesis.

    Takes as input the ARN of the Kinesis stream to which records should be
    sent, how many records should be sent at once, and how many times in a row
    records should try and fail to be sent to Kinesis before an error is
    thrown. Kinesis supports up to 500 records per batch.
    """

    def __init__(self, stream_arn, batch_size, max_retries=5):
        self.logger = create_log('kinesis_client')
        self.stream_arn = stream_arn
        self.batch_size = batch_size
        self.max_retries = max_retries

        try:
            self.kinesis_client = boto3.client(
                'kinesis', region_name=os.environ.get('AWS_REGION',
                                                      'us-east-1'))
        except ClientError as e:
            self.logger.error(
                'Could not create Kinesis client: {err}'.format(err=e))
            raise KinesisClientError(
                'Could not create Kinesis client: {err}'.format(err=e)
            ) from None

    def close(self):
        self.kinesis_client.close()

    def send_records(self, records):
        """
        Sends list of records (usually represented as Avro-encoded byte
        strings) to Kinesis in batches of size self.batch_size.
        """
        for i in range(0, len(records), self.batch_size):
            encoded_batch = records[i:i + self.batch_size]
            kinesis_records = [{'Data': record, 'PartitionKey':
                                str(int(time.time() * 1000000000))}
                               for record in encoded_batch]
            self._send_kinesis_format_records(kinesis_records, 1)

    def _send_kinesis_format_records(self, kinesis_records, call_count):
        """
        Sends list of records in Kinesis format to Kinesis. This method is
        recursively called when Kinesis fails to retrieve some of the records.
        """
        if call_count > self.max_retries:
            self.logger.error(
                'Failed to send records to Kinesis {} times in a row'.format(
                    call_count-1))
            raise KinesisClientError(
                'Failed to send records to Kinesis {} times in a row'.format(
                    call_count-1)) from None

        try:
            self.logger.info(
                'Sending ({count}) records to {arn} Kinesis stream'.format(
                    count=len(kinesis_records), arn=self.stream_arn))
            response = self.kinesis_client.put_records(
                Records=kinesis_records, StreamARN=self.stream_arn)
            if response['FailedRecordCount'] > 0:
                self.logger.warning(
                    'Failed to send {} records to Kinesis'.format(
                        response['FailedRecordCount']))
                failed_records = []
                for i in range(len(response['Records'])):
                    if 'ErrorCode' in response['Records'][i]:
                        failed_records.append(kinesis_records[i])
                self._send_kinesis_format_records(failed_records, call_count+1)
        except ClientError as e:
            self.logger.error(
                'Error sending records to Kinesis: {}'.format(e))
            raise KinesisClientError(
                'Error sending records to Kinesis: {}'.format(e)) from None


class KinesisClientError(Exception):
    def __init__(self, message=None):
        self.message = message



import bcrypt
import os

from nypl_py_utils.functions.log_helper import create_log

logger = create_log('obfuscation_helper')


def obfuscate(input):
    """
    This method obfuscates an input according to NYPL standards using bcrypt.
    For more information on the obfuscation method, see
    https://github.com/NYPL/BIC/blob/main/obfuscating-identifiers.md.

    The method returns a string and takes one input: `input` can be of any type
    but is converted to a string before being obfuscated. The obfuscation salt
    is read from the `BCRYPT_SALT` environment variable.
    """
    logger.debug('Obfuscating input \'{}\' with environment salt'.format(
        input))
    hash = bcrypt.hashpw(str(input).encode(),
                         os.environ['BCRYPT_SALT'].encode()).decode()
    return hash.split(os.environ['BCRYPT_SALT'])[-1]


def obfuscate_with_salt(input, salt):
    """
    This method is the same as `obfuscate` above but takes the obfuscation salt
    as a string input.
    """
    logger.debug('Obfuscating input \'{}\' with custom salt'.format(input))
    hash = bcrypt.hashpw(str(input).encode(), salt.encode()).decode()
    return hash.split(salt)[-1]



import os
from oauthlib.oauth2 import BackendApplicationClient, TokenExpiredError
from requests_oauthlib import OAuth2Session
from nypl_py_utils.functions.log_helper import create_log


class Oauth2ApiClient:
    """
    Client for interacting with an Oauth2 authenticated API such as NYPL's
    Platform API endpoints
    """

    def __init__(self, client_id=None, client_secret=None, base_url=None,
                 token_url=None):
        self.client_id = client_id \
            or os.environ.get('NYPL_API_CLIENT_ID', None)
        self.client_secret = client_secret \
            or os.environ.get('NYPL_API_CLIENT_SECRET', None)
        self.token_url = token_url \
            or os.environ.get('NYPL_API_TOKEN_URL', None)
        self.base_url = base_url \
            or os.environ.get('NYPL_API_BASE_URL', None)

        self.client = None
        self.token = None

        self.logger = create_log('oauth2_api_client')

    def get(self, request_path, **kwargs):
        """
        Issue an HTTP GET on the given request_path
        """
        return self._do_http_method('GET', request_path, **kwargs)

    def post(self, request_path, json, **kwargs):
        """
        Issue an HTTP POST on the given request_path with given JSON body
        """
        kwargs['json'] = json
        return self._do_http_method('POST', request_path, **kwargs)

    def patch(self, request_path, json, **kwargs):
        """
        Issue an HTTP PATCH on the given request_path with given JSON body
        """
        kwargs['json'] = json
        return self._do_http_method('PATCH', request_path, **kwargs)

    def delete(self, request_path, **kwargs):
        """
        Issue an HTTP DELETE on the given request_path
        """
        return self._do_http_method('DELETE', request_path, **kwargs)

    def _do_http_method(self, method, request_path, **kwargs):
        """
        Issue an HTTP method call on on the given request_path
        """
        if not self.client:
            self._create_oauth_client()

        url = f'{self.base_url}/{request_path}'
        self.logger.debug(f'{method} {url}')

        try:
            return self.oauth_client.request(method, url, **kwargs).json()
        except TokenExpiredError as e:
            self.logger.debug(f'TokenExpiredError encountered: {e}')
            self._generate_access_token()

            return self._do_http_method(method, request_path, **kwargs)
        except TimeoutError as e:
            self.logger.error(f'TimeoutError encountered: {e}')
            return {}

    def _create_oauth_client(self):
        """
        Creates an authenticated a OAuth2Session instance for later requests
        """
        self._generate_access_token()
        self.oauth_client = OAuth2Session(self.client_id, token=self.token)

    def _generate_access_token(self):
        """
        Fetch and store a fresh token
        """
        client = BackendApplicationClient(client_id=self.client_id)
        oauth = OAuth2Session(client=client)
        self.logger.debug(f'Refreshing token via @{self.token_url}')
        self.token = oauth.fetch_token(
            token_url=self.token_url,
            client_id=self.client_id,
            client_secret=self.client_secret
        )



import os
import pytest

from nypl_py_utils.functions.config_helper import (
    load_env_file, ConfigHelperError)

_TEST_VARIABLE_NAMES = ['TEST_STRING', 'TEST_INT', 'TEST_ENCRYPTED_VARIABLE_1',
                        'TEST_ENCRYPTED_VARIABLE_2']
_TEST_CONFIG_CONTENTS = \
    '''---
PLAINTEXT_VARIABLES:
    TEST_STRING: string-variable
    TEST_INT: 1
ENCRYPTED_VARIABLES:
    TEST_ENCRYPTED_VARIABLE_1: test-encryption-1
    TEST_ENCRYPTED_VARIABLE_2: test-encryption-2
...'''


class TestConfigHelper:

    def test_load_env_file(self, mocker):
        mock_kms_client = mocker.MagicMock()
        mock_kms_client.decrypt.side_effect = [
            'test-decryption-1', 'test-decryption-2']
        mocker.patch('nypl_py_utils.functions.config_helper.KmsClient',
                     return_value=mock_kms_client)
        mock_file_open = mocker.patch(
            'builtins.open', mocker.mock_open(read_data=_TEST_CONFIG_CONTENTS))

        for key in _TEST_VARIABLE_NAMES:
            assert key not in os.environ
        load_env_file('test-env', 'test-path/{}.yaml')

        mock_file_open.assert_called_once_with('test-path/test-env.yaml', 'r')
        mock_kms_client.decrypt.assert_has_calls([
            mocker.call('test-encryption-1'), mocker.call('test-encryption-2')]
        )
        mock_kms_client.close.assert_called_once()
        assert os.environ['TEST_STRING'] == 'string-variable'
        assert os.environ['TEST_INT'] == '1'
        assert os.environ['TEST_ENCRYPTED_VARIABLE_1'] == 'test-decryption-1'
        assert os.environ['TEST_ENCRYPTED_VARIABLE_2'] == 'test-decryption-2'

        for key in _TEST_VARIABLE_NAMES:
            if key in os.environ:
                del os.environ[key]

    def test_missing_file_error(self):
        with pytest.raises(ConfigHelperError):
            load_env_file('bad-env', 'bad-path/{}.yaml')

    def test_bad_yaml(self, mocker):
        mocker.patch(
            'builtins.open', mocker.mock_open(read_data='bad yaml: ['))
        with pytest.raises(ConfigHelperError):
            load_env_file('test-env', 'test-path/{}.not_yaml')



import os
import yaml

from nypl_py_utils import KmsClient
from nypl_py_utils.functions.log_helper import create_log

logger = create_log('config_helper')


def load_env_file(run_type, file_string):
    """
    This method loads a YAML config file containing environment variables,
    decrypts whichever are encrypted, and puts them all into os.environ as
    strings.

    It requires the YAML file to be split into a 'PLAINTEXT_VARIABLES' section
    and an 'ENCRYPTED_VARIABLES' section.
    """

    env_dict = None
    open_file = file_string.format(run_type)
    logger.info('Loading env file {}'.format(open_file))
    try:
        with open(open_file, 'r') as env_stream:
            try:
                env_dict = yaml.safe_load(env_stream)
            except yaml.YAMLError:
                logger.error('Invalid YAML file: {}'.format(open_file))
                raise ConfigHelperError(
                    'Invalid YAML file: {}'.format(open_file)) from None
    except FileNotFoundError:
        logger.error('Could not find config file {}'.format(open_file))
        raise ConfigHelperError(
            'Could not find config file {}'.format(open_file)) from None

    if env_dict:
        for key, value in env_dict.get('PLAINTEXT_VARIABLES', {}).items():
            os.environ[key] = str(value)

        kms_client = KmsClient()
        for key, value in env_dict.get('ENCRYPTED_VARIABLES', {}).items():
            os.environ[key] = kms_client.decrypt(value)
        kms_client.close()


class ConfigHelperError(Exception):
    def __init__(self, message=None):
        self.message = message



import pytest

from base64 import b64encode
from nypl_py_utils import KmsClient, KmsClientError

_TEST_ENCRYPTED_VALUE = b64encode(b'test-encrypted-value')
_TEST_DECRYPTION = {
    'KeyId': 'test-key-id',
    'Plaintext': b'test-decrypted-value',
    'EncryptionAlgorithm': 'test-encryption-algorithm',
    'ResponseMetadata': {}
}


class TestKmsClient:

    @pytest.fixture
    def test_instance(self, mocker):
        mocker.patch('boto3.client')
        return KmsClient()

    def test_decrypt(self, test_instance):
        test_instance.kms_client.decrypt.return_value = _TEST_DECRYPTION
        assert test_instance.kms_client.decrypt.called_once_with(
            CiphertextBlob=b'test-encrypted-value')
        assert test_instance.decrypt(
            _TEST_ENCRYPTED_VALUE) == 'test-decrypted-value'

    def test_base64_error(self, test_instance):
        with pytest.raises(KmsClientError):
            test_instance.decrypt('bad-b64')



import json
import pytest

from nypl_py_utils import AvroEncoder, AvroEncoderError
from requests.exceptions import ConnectTimeout

_TEST_SCHEMA = {'data': {'schema': json.dumps({
    'name': 'TestSchema',
    'type': 'record',
    'fields': [
        {
            'name': 'patron_id',
            'type': 'int'
        },
        {
            'name': 'library_branch',
            'type': ['null', 'string']
        }
    ]
})}}


class TestAvroEncoder:

    @pytest.fixture
    def test_instance(self, requests_mock):
        requests_mock.get(
            'https://test_schema_url', text=json.dumps(_TEST_SCHEMA))
        return AvroEncoder('https://test_schema_url')

    def test_get_json_schema(self, test_instance):
        assert test_instance.schema == _TEST_SCHEMA['data']['schema']

    def test_request_error(self, requests_mock):
        requests_mock.get('https://test_schema_url', exc=ConnectTimeout)
        with pytest.raises(AvroEncoderError):
            AvroEncoder('https://test_schema_url')

    def test_bad_json_error(self, requests_mock):
        requests_mock.get(
            'https://test_schema_url', text='bad json')
        with pytest.raises(AvroEncoderError):
            AvroEncoder('https://test_schema_url')

    def test_missing_key_error(self, requests_mock):
        requests_mock.get(
            'https://test_schema_url', text=json.dumps({'field': 'value'}))
        with pytest.raises(AvroEncoderError):
            AvroEncoder('https://test_schema_url')

    def test_encode_record(self, test_instance):
        TEST_RECORD = {'patron_id': 123, 'library_branch': 'aa'}
        encoded_record = test_instance.encode_record(TEST_RECORD)
        assert type(encoded_record) is bytes
        assert test_instance.decode_record(encoded_record) == TEST_RECORD

    def test_encode_record_error(self, test_instance):
        TEST_RECORD = {'patron_id': 123, 'bad_field': 'bad'}
        with pytest.raises(AvroEncoderError):
            test_instance.encode_record(TEST_RECORD)

    def test_encode_batch(self, test_instance):
        TEST_BATCH = [
            {'patron_id': 123, 'library_branch': 'aa'},
            {'patron_id': 456, 'library_branch': None},
            {'patron_id': 789, 'library_branch': 'bb'}]
        encoded_records = test_instance.encode_batch(TEST_BATCH)
        assert len(encoded_records) == len(TEST_BATCH)
        for i in range(3):
            assert type(encoded_records[i]) is bytes
            assert test_instance.decode_record(
                encoded_records[i]) == TEST_BATCH[i]

    def test_encode_batch_error(self, test_instance):
        BAD_BATCH = [
            {'patron_id': 123, 'library_branch': 'aa'},
            {'patron_id': 456, 'bad_field': 'bad'}]
        with pytest.raises(AvroEncoderError):
            test_instance.encode_batch(BAD_BATCH)

    def test_decode_record(self, test_instance):
        TEST_DECODED_RECORD = {'patron_id': 123, 'library_branch': 'aa'}
        TEST_ENCODED_RECORD = b'\xf6\x01\x02\x04aa'
        assert test_instance.decode_record(
            TEST_ENCODED_RECORD) == TEST_DECODED_RECORD

    def test_decode_record_error(self, test_instance):
        TEST_ENCODED_RECORD = b'bad-encoding'
        with pytest.raises(AvroEncoderError):
            test_instance.decode_record(TEST_ENCODED_RECORD)



import boto3
import os

from base64 import b64decode
from binascii import Error as base64Error
from botocore.exceptions import ClientError
from nypl_py_utils.functions.log_helper import create_log


class KmsClient:
    """Client for interacting with a KMS client"""

    def __init__(self):
        self.logger = create_log('kms_client')

        try:
            self.kms_client = boto3.client(
                'kms', region_name=os.environ.get('AWS_REGION', 'us-east-1'))
        except ClientError as e:
            self.logger.error(
                'Could not create KMS client: {err}'.format(err=e))
            raise KmsClientError(
                'Could not create KMS client: {err}'.format(err=e)) from None

    def close(self):
        self.kms_client.close()

    def decrypt(self, encrypted_text):
        """
        This method takes a base 64 KMS-encoded string and uses the KMS client
        to decrypt it into a usable string.
        """
        self.logger.debug('Decrypting \'{}\''.format(encrypted_text))
        try:
            decoded_text = b64decode(encrypted_text)
            return self.kms_client.decrypt(CiphertextBlob=decoded_text)[
                'Plaintext'].decode('utf-8')
        except (ClientError, base64Error, TypeError) as e:
            self.logger.error('Could not decrypt \'{val}\': {err}'.format(
                val=encrypted_text, err=e))
            raise KmsClientError('Could not decrypt \'{val}\': {err}'.format(
                val=encrypted_text, err=e)) from None


class KmsClientError(Exception):
    def __init__(self, message=None):
        self.message = message



import redshift_connector

from botocore.exceptions import ClientError
from nypl_py_utils.functions.log_helper import create_log


class RedshiftClient:
    """Client for managing connections to Redshift"""

    def __init__(self, host, database, user, password):
        self.logger = create_log('redshift_client')
        self.conn = None
        self.host = host
        self.database = database
        self.user = user
        self.password = password

    def connect(self):
        """Connects to a Redshift database using the given credentials"""
        self.logger.info('Connecting to {} database'.format(self.database))
        try:
            self.conn = redshift_connector.connect(
                host=self.host,
                database=self.database,
                user=self.user,
                password=self.password)
        except ClientError as e:
            self.logger.error(
                'Error connecting to {name} database: {error}'.format(
                    name=self.database, error=e))
            raise RedshiftClientError(
                'Error connecting to {name} database: {error}'.format(
                    name=self.database, error=e)) from None

    def execute_query(self, query, dataframe=False):
        """
        Executes an arbitrary query against the given database connection.

        Parameters
        ----------
        query: str
            The query to execute
        dataframe: bool, optional
            Whether the data will be returned as a pandas DataFrame. Defaults
            to False, which means the data is returned as a list of tuples.

        Returns
        -------
        None or sequence
            None if is_write_query is True. A list of tuples or a pandas
            DataFrame (based on the dataframe input) if is_write_query is
            False.
        """
        self.logger.info('Querying {} database'.format(self.database))
        self.logger.debug('Executing query {}'.format(query))
        try:
            cursor = self.conn.cursor()
            cursor.execute(query)
            if dataframe:
                return cursor.fetch_dataframe()
            else:
                return cursor.fetchall()
        except Exception as e:
            self.conn.rollback()
            self.logger.error(
                ('Error executing {name} database query \'{query}\': {error}')
                .format(name=self.database, query=query, error=e))
            raise RedshiftClientError(
                ('Error executing {name} database query \'{query}\': {error}')
                .format(name=self.database, query=query, error=e)) from None
        finally:
            cursor.close()

    def close_connection(self):
        """Closes the database connection"""
        self.logger.debug('Closing {} database connection'.format(
            self.database))
        self.conn.close()


class RedshiftClientError(Exception):
    def __init__(self, message=None):
        self.message = message



import pytest

from nypl_py_utils import MySQLClient, MySQLClientError


class TestMySQLClient:

    @pytest.fixture
    def mock_mysql_conn(self, mocker):
        return mocker.patch('mysql.connector.connect')

    @pytest.fixture
    def test_instance(self):
        return MySQLClient('test_host', 'test_port', 'test_database',
                           'test_user', 'test_password')

    def test_connect(self, mock_mysql_conn, test_instance):
        test_instance.connect()
        mock_mysql_conn.assert_called_once_with(host='test_host',
                                                port='test_port',
                                                database='test_database',
                                                user='test_user',
                                                password='test_password')

    def test_execute_read_query(self, mock_mysql_conn, test_instance, mocker):
        test_instance.connect()

        mock_cursor = mocker.MagicMock()
        mock_cursor.fetchall.return_value = [(1, 2, 3), ('a', 'b', 'c')]
        test_instance.conn.cursor.return_value = mock_cursor

        assert test_instance.execute_query(
            'test query') == [(1, 2, 3), ('a', 'b', 'c')]
        test_instance.conn.cursor.called_once_with(dictionary=False)
        mock_cursor.execute.assert_called_once_with('test query', None)
        mock_cursor.close.assert_called_once()

    def test_execute_dictionary_read_query(self, mock_mysql_conn,
                                           test_instance, mocker):
        test_instance.connect()

        mock_cursor = mocker.MagicMock()
        mock_cursor.fetchall.return_value = [
            {'col1': 1, 'col2': 'a'},
            {'col1': 2, 'col2': 'b'},
            {'col1': 3, 'col2': 'c'}]
        test_instance.conn.cursor.return_value = mock_cursor

        assert test_instance.execute_query(
            'test query', dictionary=True) == [{'col1': 1, 'col2': 'a'},
                                               {'col1': 2, 'col2': 'b'},
                                               {'col1': 3, 'col2': 'c'}]
        test_instance.conn.cursor.called_once_with(dictionary=True)
        mock_cursor.execute.assert_called_once_with('test query', None)
        mock_cursor.close.assert_called_once()

    def test_execute_write_query(self, mock_mysql_conn, test_instance, mocker):
        test_instance.connect()

        mock_cursor = mocker.MagicMock()
        test_instance.conn.cursor.return_value = mock_cursor

        assert test_instance.execute_query(
            'test query', is_write_query=True) is None
        test_instance.conn.cursor.called_once_with(dictionary=False)
        mock_cursor.execute.assert_called_once_with('test query', None)
        test_instance.conn.commit.called_once()
        mock_cursor.close.assert_called_once()

    def test_execute_write_query_with_params(self, mock_mysql_conn,
                                             test_instance, mocker):
        test_instance.connect()

        mock_cursor = mocker.MagicMock()
        test_instance.conn.cursor.return_value = mock_cursor

        assert test_instance.execute_query(
            'test query %s %s', is_write_query=True,
            query_params=('a', 1)) is None
        test_instance.conn.cursor.called_once_with(dictionary=False)
        mock_cursor.execute.assert_called_once_with('test query %s %s',
                                                    ('a', 1))
        test_instance.conn.commit.called_once()
        mock_cursor.close.assert_called_once()

    def test_execute_query_with_exception(
            self, mock_mysql_conn, test_instance, mocker):
        test_instance.connect()

        mock_cursor = mocker.MagicMock()
        mock_cursor.execute.side_effect = Exception()
        test_instance.conn.cursor.return_value = mock_cursor

        with pytest.raises(MySQLClientError):
            test_instance.execute_query('test query')

        test_instance.conn.rollback.assert_called_once()
        mock_cursor.close.assert_called_once()

    def test_close_connection(self, mock_mysql_conn, test_instance):
        test_instance.connect()
        test_instance.close_connection()
        test_instance.conn.close.assert_called_once()



import boto3
import json
import os

from botocore.exceptions import ClientError
from io import BytesIO
from nypl_py_utils.functions.log_helper import create_log


class S3Client:
    """
    Client for fetching and setting an AWS S3 file.

    Takes as input the name of the S3 bucket and resource to be fetched/set.
    """

    def __init__(self, bucket, resource):
        self.logger = create_log('s3_client')
        self.bucket = bucket
        self.resource = resource

        try:
            self.s3_client = boto3.client(
                's3', region_name=os.environ.get('AWS_REGION', 'us-east-1'))
        except ClientError as e:
            self.logger.error(
                'Could not create S3 client: {err}'.format(err=e))
            raise S3ClientError(
                'Could not create S3 client: {err}'.format(err=e)) from None

    def close(self):
        self.s3_client.close()

    def fetch_cache(self):
        """Fetches a JSON file from S3 and returns the resulting dictionary"""
        self.logger.info('Fetching {file} from S3 bucket {bucket}'.format(
            file=self.resource, bucket=self.bucket))
        try:
            output_stream = BytesIO()
            self.s3_client.download_fileobj(
                self.bucket, self.resource, output_stream)
            return json.loads(output_stream.getvalue())
        except ClientError as e:
            self.logger.error(
                'Error retrieving {file} from S3 bucket {bucket}: {error}'
                .format(file=self.resource, bucket=self.bucket, error=e))
            raise S3ClientError(
                'Error retrieving {file} from S3 bucket {bucket}: {error}'
                .format(file=self.resource, bucket=self.bucket, error=e)
            ) from None

    def set_cache(self, state):
        """Writes a dictionary to JSON and uploads the resulting file to S3"""
        self.logger.info(
            'Setting {file} in S3 bucket {bucket} to {state}'.format(
                file=self.resource, bucket=self.bucket, state=state))
        try:
            input_stream = BytesIO(json.dumps(state).encode())
            self.s3_client.upload_fileobj(
                input_stream, self.bucket, self.resource)
        except ClientError as e:
            self.logger.error(
                'Error uploading {file} to S3 bucket {bucket}: {error}'
                .format(file=self.resource, bucket=self.bucket, error=e))
            raise S3ClientError(
                'Error uploading {file} to S3 bucket {bucket}: {error}'
                .format(file=self.resource, bucket=self.bucket, error=e)
            ) from None


class S3ClientError(Exception):
    def __init__(self, message=None):
        self.message = message



import os

from nypl_py_utils.functions.obfuscation_helper import (obfuscate,
                                                        obfuscate_with_salt)

_TEST_SALT_1 = '$2a$10$8AvAPrrUsmlBa50qgc683e'
_TEST_SALT_2 = '$2b$12$iuSSdD6F/nJ1GSXzesM8sO'


class TestObfuscationHelper:

    def test_obfuscation_with_environment_variable(self):
        os.environ['BCRYPT_SALT'] = _TEST_SALT_1
        assert obfuscate('test_input') == 'UPMawmdZfleeSg5REsZbLbAivWl97O6'
        del os.environ['BCRYPT_SALT']

    def test_obfuscation_with_custom_salt(self):
        assert (obfuscate_with_salt('test_input', _TEST_SALT_2) ==
                'SUXLCHnsRVt4Vj1PyP9KPEqADxtUj5.')
